<?php

namespace Drupal\ephoto_dam\Plugin\Filter;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the plugin.
 *
 * @Filter(
 *   title = @Translation("Ephoto Dam"),
 *   id = "ephoto_dam",
 *   description = @Translation("Activate the use of the Ephoto Dam filter"),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_MARKUP_LANGUAGE
 * )
 */
class EphotoDam extends FilterBase implements ContainerFactoryPluginInterface {

    /**
     * Default settings
     *
     * @const
     */	
    private const DEFAULT_SETTINGS = [
        'ephoto_dam',
        'align',
        'src',
        'embed',
        'width',
        'height',
        'alt',
        'caption',
    ];
    
    /**
     * ephoto_dam constructor.
     *
     * @param array $configuration Plugin configuration.
     * @param string $plugin_id Plugin ID.
     * @param mixed $plugin_definition Plugin definition.
     */
    public function __construct(array $configuration, $plugin_id, $plugin_definition) {	
        parent::__construct($configuration, $plugin_id, $plugin_definition);
    }

    /**
     * {@inheritdoc}
     */
    public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
        return new static($configuration, $plugin_id, $plugin_definition);
    }

    /**
     * {@inheritdoc}
     */
    public function process($text, $langcode) {
        $result = new FilterProcessResult($text);
        
        foreach ($this->getValidMatches($text) as $source => $values) {

            if($values['ephoto_dam'] === 'image') {
                $html = $this->_renderImage((object) $values);
            
            } else {
                $html = $this->_renderEmbed((object) $values);
            }
            
            $text = str_replace($source, $html, $text);
            
            $result->setAttachments(['library' => ['ephoto_dam/display']]);
        }

        $result->setProcessedText($text);

        return $result;
    }
    
    /**
     * Render Image code html
     *
     * @param object $values Values of item
     * @return string
     */	
    private function _renderImage($values) {
        $html = '';

        if($values->caption) {
            $html .= '<figure class="caption caption-img ephoto-dam-align-' . $values->align . '" role="group">';
        }
        
        $html .= '<img';
        $html .= ' alt="' . $values->alt . '"';
        $html .= ' src="' . $values->src . '"';
        $html .= ' width="' . $values->width . '"';
        $html .= ' height="' . $values->height . '"';
        $html .= ' class="' . ($values->zoom ? 'ephoto-dam-zoom' : '');
        
        if(!$values->caption) $html .= ' align-' . $values->align;
        
        $html .= '">';

        if($values->caption) {
            $html .= '<figcaption>' . $values->caption_data . '</figcaption>';
            $html .= '</figure>';
        }
        
        return $html;
    }
    
    /**
     * Render Embed code html
     *
     * @param object $values Values of item
     * @return string
     */	
    private function _renderEmbed($values) {
        $html = '';
        
        if($values->caption) {
            $html .= '<figure class="caption caption-img ephoto-dam-align-' . $values->align . '" role="group">';
        }
        
        $html .= '<iframe scrolling="no" allow="autoplay" allowfullscreen="true" frameborder="0"';
        $html .= ' alt="' . $values->alt . '"';
        $html .= ' src="' . $values->embed . '"';
        $html .= ' width="' . $values->width . '"';
        $html .= ' height="' . $values->height . '"';
        
        if(!$values->caption) $html .= ' class="align-' . $values->align . '"';
        
        $html .= '></iframe>';
        
        if($values->caption) {
            $html .= '<figcaption>' . $values->caption_data . '</figcaption>';
            $html .= '</figure>';
        }

        return $html;
    }
    
    /**
     * Get all valid matches in the WYSIWYG.
     *
     * @param string $text The text to check for WYSIWYG matches.
     * @return array An array of data from the text keyed by the text content.
     */
    protected function getValidMatches($text) {
        // Use a look ahead to match the capture groups in any order.
        if (!preg_match_all('/(<p>)?(?<json>{(?=.*ephoto_dam\b)(?=.*align\b)(?=.*src\b)(?=.*embed)(?=.*width)(?=.*height)(?=.*alt)(?=.*caption)(?=.*caption_data)(?=.*zoom)(.*)})(<\/p>)?/', $text, $matches)) {
            return [];
        }

        $valid_matches = [];
        
        foreach ($matches['json'] as $delta => $match) {
            // Ensure the JSON string is valid.
            $embed_data = json_decode($match, true);

            if (!$embed_data || !is_array($embed_data)) {
                continue;
            }

            if ($this->isValidSettings($embed_data)) {
                $valid_matches[$matches[0][$delta]] = $embed_data;
            }
        }

        return $valid_matches;
    }
    
    /**
     * Check if the given settings are valid.
     *
     * @param array $settings Settings to validate.
     * @return bool If the required settings are present.
     */
    protected function isValidSettings($settings) {
        foreach (self::DEFAULT_SETTINGS as $setting) {
            if (!isset($settings[$setting])) {
                return false;
            }
        }

        return true;
    }	
}
